@@ -19,7 +19,7 @@
 
 import fnmatch
 import re
-from typing import List, Optional, Union
+from typing import Callable, Iterator, List, Optional, TextIO, Union
 
 from .reformatting import Editor
 
@@ -28,10 +28,10 @@ from .reformatting import Editor
 # [[<package>][ <archlist>][ <type>]: ]<lintian-tag>[ [*]<lintian-info>[*]]
 
 
-VALID_TYPES = ['udeb', 'source', 'binary']
+VALID_TYPES = ["udeb", "source", "binary"]
 
 
-def _create_matcher(value):
+def _create_matcher(value: Optional[str]) -> Callable[[str], bool]:
     if value:
         p = re.compile(fnmatch.translate(value))
         return lambda x: bool(p.match(x))
@@ -40,10 +40,14 @@ def _create_matcher(value):
 
 
 class LintianOverride:
-
-    def __init__(self, package: Optional[str] = None,
-                 archlist: Optional[List[str]] = None,
-                 type=None, tag=None, info=None):
+    def __init__(
+        self,
+        package: Optional[str] = None,
+        archlist: Optional[List[str]] = None,
+        type: Optional[str] = None,
+        tag: Optional[str] = None,
+        info: Optional[str] = None,
+    ) -> None:
         self.package = package
         self.archlist = archlist
         if type is not None and type not in VALID_TYPES:
@@ -54,66 +58,66 @@ class LintianOverride:
         self.info = info
         self._info_match = _create_matcher(self.info)
 
-    def __repr__(self):
-        return (
-            "{}(package={!r}, archlist={!r}, type={!r}, tag={!r}, info={!r})"
-            .format(
-                type(self).__name__, self.package, self.archlist, self.type,
-                self.tag, self.info))
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}(package={self.package!r}, archlist={self.archlist!r}, type={self.type!r}, tag={self.tag!r}, info={self.info!r})"
 
-    def matches(self, package: Optional[str] = None, tag: Optional[str] = None,
-                info: Optional[str] = None, arch: Optional[str] = None,
-                type: Optional[str] = None) -> bool:
-        if (self.package is not None and package is not None
-                and self.package != package):
+    def matches(
+        self,
+        package: Optional[str] = None,
+        tag: Optional[str] = None,
+        info: Optional[str] = None,
+        arch: Optional[str] = None,
+        type: Optional[str] = None,
+    ) -> bool:
+        if self.package is not None and package is not None and self.package != package:
             return False
         if self.type is not None and type is not None and self.type != type:
             return False
-        if (self.tag is not None and tag is not None
-                and not self._tag_match(tag)):
+        if self.tag is not None and tag is not None and not self._tag_match(tag):
             return False
-        if (self.info is not None and info is not None
-                and not self._info_match(info)):
+        if self.info is not None and info is not None and not self._info_match(info):
             return False
         # TODO(jelmer): wildcards in the arch list?
         if self.archlist and arch is not None and arch not in self.archlist:
             return False
         return True
 
-    def __eq__(self, other):
-        return (isinstance(other, type(self))
-                and self.package == other.package
-                and self.archlist == other.archlist
-                and self.type == other.type
-                and self.tag == other.tag
-                and self.info == other.info)
+    def __eq__(self, other: object) -> bool:
+        return (
+            isinstance(other, type(self))
+            and self.package == other.package
+            and self.archlist == other.archlist
+            and self.type == other.type
+            and self.tag == other.tag
+            and self.info == other.info
+        )
 
 
 class LintianOverridesEditor(Editor[List[Union[str, LintianOverride]], str]):
-
-    def _parse(self, content):
+    def _parse(self, content: str) -> List[Union[str, LintianOverride]]:
         """Parse the specified bytestring and returned parsed object."""
-        ret = []
+        ret: List[Union[str, LintianOverride]] = []
         for line in content.splitlines(True):
-            if line.startswith('#') or not line.strip():
+            if line.startswith("#") or not line.strip():
                 ret.append(line)
             else:
                 ret.append(parse_override(line))
         return ret
 
     @property
-    def lines(self):
+    def lines(self) -> List[Union[str, LintianOverride]]:
         return self._parsed or []
 
     @property
-    def overrides(self):
-        return [entry for entry in self.lines
-                if isinstance(entry, LintianOverride)]
+    def overrides(self) -> List[LintianOverride]:
+        return [entry for entry in self.lines if isinstance(entry, LintianOverride)]
 
-    def _nonexistent(self):
-        return None
+    def _nonexistent(self) -> List[Union[str, LintianOverride]]:
+        return []
 
-    def override_exists(self, tag, info=None, package=None):
+    def override_exists(
+        self, tag: str, info: Optional[str] = None, package: Optional[str] = None
+    ) -> bool:
         """Check if a particular override exists.
 
         Args:
@@ -126,7 +130,9 @@ class LintianOverridesEditor(Editor[List[Union[str, LintianOverride]], str]):
                 return True
         return False
 
-    def _format(self, parsed):
+    def _format(
+        self, parsed: Optional[List[Union[str, LintianOverride]]]
+    ) -> Optional[str]:
         """Serialize the parsed object."""
         if self._parsed is None:
             return None
@@ -136,11 +142,11 @@ class LintianOverridesEditor(Editor[List[Union[str, LintianOverride]], str]):
                 ret.append(entry)
             else:
                 ret.append(serialize_override(entry))
-        return ''.join(ret)
+        return "".join(ret)
 
 
 def parse_override(line: str) -> LintianOverride:
-    """Parse an override line
+    """Parse an override line.
 
     Args:
       line: Line to parse
@@ -154,19 +160,19 @@ def parse_override(line: str) -> LintianOverride:
     archlist = None
     package = None
     type = None
-    if ': ' in line:
-        origin, issue = line.split(': ', 1)
+    if ": " in line:
+        origin, issue = line.split(": ", 1)
         while origin:
             origin = origin.strip()
-            if origin.startswith('['):
-                archs, origin = origin[1:].split(']', 1)
+            if origin.startswith("["):
+                archs, origin = origin[1:].split("]", 1)
                 archlist = archs.strip().split()
             else:
                 try:
-                    field, origin = origin.split(' ', 1)
+                    field, origin = origin.split(" ", 1)
                 except ValueError:
                     field = origin
-                    origin = ''
+                    origin = ""
                 if field in VALID_TYPES:
                     type = field
                 else:
@@ -179,10 +185,11 @@ def parse_override(line: str) -> LintianOverride:
         tag = issue
         info = None
     return LintianOverride(
-        package=package, archlist=archlist, type=type, tag=tag, info=info)
+        package=package, archlist=archlist, type=type, tag=tag, info=info
+    )
 
 
-def serialize_override(override):
+def serialize_override(override: LintianOverride) -> str:
     """Serialize an override.
 
     Args:
@@ -194,23 +201,22 @@ def serialize_override(override):
     if override.package:
         origin.append(override.package)
     if override.archlist:
-        origin.append('[' + ' '.join(override.archlist) + ']')
+        origin.append("[" + " ".join(override.archlist) + "]")
     if override.type:
         origin.append(override.type)
     if origin:
-        line = ' '.join(origin) + ': ' + override.tag
+        line = " ".join(origin) + ": " + (override.tag or "")
     else:
-        line = override.tag
+        line = override.tag or ""
     if override.info:
-        line += ' ' + override.info
-    return line + '\n'
+        line += " " + override.info
+    return line + "\n"
 
 
-def iter_overrides(f):
-    """Iterate over overrides in a file.
-    """
+def iter_overrides(f: TextIO) -> Iterator[LintianOverride]:
+    """Iterate over overrides in a file."""
     for line in f.readlines():
-        if line.startswith('#') or not line.strip():
+        if line.startswith("#") or not line.strip():
             pass
         else:
             yield parse_override(line)
